@@ -1,22 +1,22 @@
-class Test::Builder::TestPlan-0.1.0
+class Test::Builder::TestPlan-0.1.1
 {
-    has Int $.expect;
+    has Int $:expect;
 
     submethod BUILD ( ?$expect = 0 )
     {
         fail "Invalid or missing plan" unless $expect;
-        $.expect = $expect;
+        $:expect = $expect;
     }
 
     method header returns Str
     {
-        return "1..$.expect";
+        return "1..$:expect";
     }
 
     method footer returns Str (Int $run)
     {
-        return '' if $run == $.expect;
-        return "Expected $self.expect but ran $run";
+        return '' if $run == $:expect;
+        return "Expected $.expect but ran $run";
     }
 }
 
