@@ -21,11 +21,12 @@ import Pugs.Embed.Haskell
 import Pugs.Embed.Parrot
 -- import Pugs.Embed.Ponie
 
+evalEmbedded :: String -> String -> IO ()
 evalEmbedded "Parrot"  code = do
     evalParrot code
-evalEmbedded "Haskell" code = do
+{- evalEmbedded "Haskell" code = do
     evalHaskell code
-    return ()
+    return () -}
 evalEmbedded "Perl5" code = do
     interp <- initPerl5 ""
     evalPerl5 code
