@@ -25,7 +25,11 @@ Functions of notable interest:
 * 'apply' and 'doApply' (the guts of function application)
 -}
 
-module Pugs.Eval where
+module Pugs.Eval (
+    evaluate,
+    emptyEnv, evaluateMain,
+    enterLValue, enterRValue,
+) where
 import Pugs.Config
 import Pugs.Internals
 import Prelude hiding ( exp )
@@ -35,23 +39,25 @@ import Pugs.AST
 import Pugs.Junc
 import Pugs.Bind
 import Pugs.Prim
-import Pugs.Prim.Match (op2Match)
 import Pugs.Prim.List (op0Zip)
 import Pugs.Context
 import Pugs.Monads
 import Pugs.Pretty
 import Pugs.Types
-import Pugs.Prim.Eval (retEvalResult)
+import Pugs.Prim.List (op2Fold)
 import Pugs.External
 
--- |Construct a new, initially empty 'Env' (evaluation environment).
--- Used in 'Main.doParse', 'Main.doParseWith' and 'Pugs.Run.prepareEnv'.
--- Of these, only 'Pugs.Run.prepareEnv' seems to make use of the second
--- argument.
+{-|
+Construct a new, initially empty 'Env' (evaluation environment).
+
+Used in 'Main.doParse', 'Main.doParseWith' and 'Pugs.Run.prepareEnv'.
+Of these, only 'Pugs.Run.prepareEnv' seems to make use of the second
+argument.  See 'Pugs.Prims.initSyms'
+-}
 emptyEnv :: (MonadIO m, MonadSTM m) 
          => String             -- ^ Name associated with the environment
          -> [STM (Pad -> Pad)] -- ^ List of 'Pad'-mutating transactions used
-                               -- to declare an initial set of global vars
+                               --     to declare an initial set of global vars
          -> m Env
 emptyEnv name genPad = do
     uniq <- liftIO newUnique
@@ -69,6 +75,7 @@ emptyEnv name genPad = do
             , envClasses = initTree
             , envEval    = evaluate
             , envCaller  = Nothing
+            , envOuter   = Nothing
             , envDepth   = 0
             , envID      = uniq
             , envBody    = Val undef
@@ -79,7 +86,7 @@ emptyEnv name genPad = do
 
 -- Evaluation ---------------------------------------------------------------
 
--- debug :: (Pretty a) => String -> String -> a -> Eval ()
+debug :: Pretty a => String -> (String -> String) -> String -> a -> Eval ()
 debug key fun str a = do
     rv <- asks envDebug
     case rv of
@@ -96,14 +103,21 @@ debug key fun str a = do
 
 evaluateMain :: Exp -> Eval Val
 evaluateMain exp = do
-    val     <- resetT $ evaluate exp
-    endAV   <- evalVar "@*END"
-    subs    <- fromVals endAV
+    -- S04: INIT {...}*      at run time, ASAP
+    initAV   <- evalVar "@?INIT"
+    initSubs <- fromVals initAV
     enterContext CxtVoid $ do
-        mapM_ evalExp [ App (Val sub) [] [] | sub <- subs ]
+        mapM_ evalExp [ App (Val sub) [] [] | sub <- initSubs ]
+    -- The main runtime
+    val      <- resetT $ evaluate exp
+    -- S04: END {...}       at run time, ALAP
+    endAV    <- evalVar "@*END"
+    endSubs  <- fromVals endAV
+    enterContext CxtVoid $ do
+        mapM_ evalExp [ App (Val sub) [] [] | sub <- endSubs ]
     return val
 
--- |Evaluate an expression. This function mostly just delegates to 'reduce'.
+-- | Evaluate an expression. This function mostly just delegates to 'reduce'.
 evaluate :: Exp -- ^ The expression to evaluate
          -> Eval Val
 evaluate (Val val) = evalVal val
@@ -130,9 +144,6 @@ findSyms name = do
                 Nothing -> return []
     return $ concat (concat syms)
 
-enterEvalContext :: Cxt -> Exp -> Eval Val
-enterEvalContext cxt = enterContext cxt . evalExp
-
 -- Reduction ---------------------------------------------------------------
 
 retVal :: Val -> Eval Val
@@ -144,6 +155,7 @@ retItem val = do
         (retVal $ VList [val])
         (retVal $ val)
 
+addGlobalSym :: (Pad -> Pad) -> Eval ()
 addGlobalSym newSym = do
     glob <- asks envGlobal
     liftSTM $ do
@@ -152,7 +164,9 @@ addGlobalSym newSym = do
 
 trapVal :: Val -> Eval a -> Eval a
 trapVal val action = case val of
-    VError str exp  -> retError str exp
+    VError str posList -> do
+        pos <- asks envPos
+        shiftT . const . return $ VError str (pos:posList)
     VControl c      -> retControl c
     _               -> action
 
@@ -163,14 +177,19 @@ evalVar name = do
         Just var -> readRef var
         Nothing  -> retError "Undeclared variable" name
 
+enterLValue :: Eval a -> Eval a
 enterLValue = local (\e -> e{ envLValue = True })
+enterRValue :: Eval a -> Eval a
 enterRValue = local (\e -> e{ envLValue = False })
 
 findVar :: Var -> Eval (Maybe VRef)
 findVar name = do
     rv <- findVarRef name
     case rv of
-        Nothing  -> return Nothing
+        Nothing  -> case name of
+            ('&':_) -> maybeM (findSub name [] []) $ \sub -> do
+                return $ codeRef sub
+            _ -> return Nothing
         Just ref -> fmap Just $ liftSTM (readTVar ref)
 
 findVarRef :: Var -> Eval (Maybe (TVar VRef))
@@ -179,9 +198,16 @@ findVarRef name
     , Just (sig, "") <- breakOnGlue "CALLER" package = do
         maybeCaller <- asks envCaller
         case maybeCaller of
-            Just caller -> local (const caller) $ do
+            Just env -> local (const env) $ do
                 findVarRef (sig ++ name')
             Nothing -> retError "cannot access CALLER:: in top level" name
+    | Just (package, name') <- breakOnGlue "::" name
+    , Just (sig, "") <- breakOnGlue "OUTER" package = do
+        maybeOuter <- asks envOuter
+        case maybeOuter of
+            Just env -> local (const env) $ do
+                findVarRef (sig ++ name')
+            Nothing -> retError "cannot access OUTER:: in top level" name
     | ('$':'?':_) <- name = do
         rv  <- getMagical name
         case rv of
@@ -191,6 +217,7 @@ findVarRef name
                 return $ Just tvar
     | otherwise = doFindVarRef name
     where
+    doFindVarRef :: Var -> Eval (Maybe (TVar VRef))
     doFindVarRef name = do
         callCC $ \foundIt -> do
             lexSym <- fmap (findSym name . envLexical) ask
@@ -202,11 +229,15 @@ findVarRef name
             when (isJust globSym) $ foundIt globSym
             return Nothing
 
+posSym :: Value a => (Pos -> a) -> Eval (Maybe Val)
 posSym f = fmap (Just . castV . f) $ asks envPos
+constSym :: String -> Eval (Maybe Val)
 constSym = return . Just . castV
 
--- |Evaluate the 'magical' variable associated with a given name. Returns 
--- Nothing if the name does not match a known magical.
+{-|
+Evaluate the \'magical\' variable associated with a given name. Returns 
+@Nothing@ if the name does not match a known magical.
+-}
 getMagical :: String -- ^ Name of the magical var to evaluate
            -> Eval (Maybe Val)
 getMagical "$?FILE"     = posSym posName
@@ -215,27 +246,13 @@ getMagical "$?COLUMN"   = posSym posBeginColumn
 getMagical "$?POSITION" = posSym pretty
 getMagical "$?MODULE"   = constSym "main"
 getMagical "$?OS"       = constSym $ getConfig "osname"
+getMagical "$?CLASS"    = fmap (Just . VType . mkType) (asks envPackage)
+getMagical "$?PACKAGE"  = fmap (Just . VType . mkType) (asks envPackage)
+getMagical "$?ROLE"     = fmap (Just . VType . mkType) (asks envPackage)
 getMagical _            = return Nothing
 
--- |Reduce an expression into its value. This is the workhorse of "Eval".
-reduce :: Exp -- ^ The expression to reduce
-       -> Eval Val
-
--- Reduction for mutables
-reduce (Val v@(VRef var)) = do
-    lv <- asks envLValue
-    if lv then retVal v else do
-    rv <- readRef var
-    retVal rv
-
--- Reduction for constants
-reduce (Val v) = retVal v
-
--- Reduction for variables
-reduce (Var name) = do
-    v <- findVar name
-    if isNothing v then retError "Undeclared variable" name else do
-    let ref = fromJust v
+evalRef :: VRef -> Eval Val
+evalRef ref = do
     if refType ref == (mkType "Thunk") then forceRef ref else do
     val <- callCC $ \esc -> do
         cxt <- asks envContext
@@ -262,6 +279,26 @@ reduce (Var name) = do
         return $ castV ref
     retVal val
 
+-- | Reduce an expression into its value. This is the workhorse of "Pugs.Eval".
+reduce :: Exp -- ^ The expression to reduce
+       -> Eval Val
+
+-- Reduction for mutables
+reduce (Val v@(VRef var)) = do
+    lv <- asks envLValue
+    if lv then retVal v else do
+    rv <- readRef var
+    retVal rv
+
+-- Reduction for constants
+reduce (Val v) = retVal v
+
+-- Reduction for variables
+reduce (Var name) = do
+    v <- findVar name
+    if isNothing v then retError "Undeclared variable" name else do
+    evalRef (fromJust v)
+
 reduce (Stmts this rest) | Noop <- unwrap rest = reduce this
 reduce (Stmts this rest) | Noop <- unwrap this = reduce rest
 
@@ -330,8 +367,19 @@ reduce exp@(Syn name exps) = case name of
     "sub" -> do
         let [exp] = exps
         (VCode sub) <- enterEvalContext (cxtItem "Code") exp
-        lex <- asks envLexical
-        retVal $ VCode sub{ subPad = lex }
+        env     <- ask
+        cont    <- if subType sub /= SubCoroutine then return Nothing else liftSTM $ do
+            tvar <- newTVar undefined
+            let thunk = MkThunk . fix $ \redo -> do
+                evalExp $ subBody sub
+                liftSTM $ writeTVar tvar thunk
+                redo
+            writeTVar tvar thunk
+            return $ Just tvar
+        retVal $ VCode sub
+            { subEnv  = Just (env{ envStash = "" })
+            , subCont = cont
+            }
     "if" -> doCond id 
     "unless" -> doCond not
     "for" -> do
@@ -351,29 +399,38 @@ reduce exp@(Syn name exps) = case name of
                 let (these, rest) = arity `splitAt` vs
                 genSymCC "&next" $ \symNext -> do
                     genSymPrim "&redo" (const $ runBody vs sub') $ \symRedo -> do
-                        apply sub'{ subPad = symRedo . symNext $ subPad sub' } [] $
+                        apply (updateSubPad sub' (symRedo . symNext)) [] $
                             map (Val . VRef . MkRef) these
                 runBody rest sub'
         genSymCC "&last" $ \symLast -> do
             let munge sub | subParams sub == [defaultArrayParam] =
                     munge sub{ subParams = [defaultScalarParam] }
-                munge sub = sub{ subPad = symLast $ subPad sub }
+                munge sub = updateSubPad sub symLast
             runBody elms $ munge sub
+    "gather" -> do
+        let [exp] = exps
+        sub     <- fromVal =<< evalExp exp
+        av      <- newArray []
+        symTake <- genSym "@?TAKE" (MkRef av)
+        apply (updateSubPad sub symTake) [] []
+        fmap VList $ readIVar av
     "loop" -> do
         let [pre, cond, post, body] = case exps of { [_] -> exps'; _ -> exps }
             exps' = [emptyExp, Val (VBool True), emptyExp] ++ exps
+            evalCond | unwrap cond == Noop = return True
+                     | otherwise = fromVal =<< enterEvalContext (cxtItem "Bool") cond
         evalExp pre
-        enterLoop . fix $ \runBody -> do
+        vb  <- evalCond
+        if not vb then retEmpty else do
+        genSymCC "&last" $ \symLast -> enterLex [symLast] $ fix $ \runBody -> do
             genSymPrim "&redo" (const $ runBody) $ \symRedo -> do
-            valBody <- enterLex [symRedo] $ evalExp body
-            valPost <- evalExp post
-            vBool   <- enterEvalContext (cxtItem "Bool") cond
-            vb      <- fromVal vBool
-            trapVal valBody $ do
-                trapVal valPost $ do
-                    if vb
-                        then runBody
-                        else retVal valBody
+            let runNext = do
+                valPost <- evalExp post
+                vb      <- evalCond
+                trapVal valPost $ if vb then runBody else retEmpty
+            genSymPrim "&next" (const $ runNext) $ \symNext -> do
+            valBody <- enterLex [symRedo, symNext] $ evalExp body
+            trapVal valBody $ runNext
     "given" -> do
         let [topic, body] = exps
         vtopic <- fromVal =<< enterLValue (enterEvalContext cxtItemAny topic)
@@ -382,9 +439,7 @@ reduce exp@(Syn name exps) = case name of
         let [match, body] = exps
         break  <- evalVar "&?BLOCK_EXIT"
         vbreak <- fromVal break
-        match  <- reduce match
-        topic  <- evalVar "$_"
-        result <- op2Match topic match
+        result <- reduce (App (Var "&infix:~~") [(Var "$_"), match] [])
         rb     <- fromVal result
         if rb
             then enterWhen (subBody vbreak) $ apply vbreak [body] []
@@ -503,6 +558,34 @@ reduce exp@(Syn name exps) = case name of
         -- error $ show list
         -- elms    <- mapM fromVal list -- flatten
         retVal $ VList (drop idx $ list)
+    "@{}" -> do
+        let [exp] = exps
+        val     <- enterEvalContext (cxtItem "Hash") exp
+        ivar    <- doArray val IArray
+        evalRef (MkRef ivar)
+    "%{}" -> do
+        let [exp] = exps
+        val     <- enterEvalContext (cxtItem "Hash") exp
+        ivar    <- doHash val IHash
+        evalRef (MkRef ivar)
+    "&{}" -> do
+        let [exp] = exps
+        val     <- enterEvalContext (cxtItem "Hash") exp
+        sub     <- fromVal val
+        return $ VCode sub
+    "${}" -> do
+        let [exp] = exps
+        val     <- enterEvalContext (cxtItem "Hash") exp
+        ref     <- fromVal val
+        evalRef ref
+    sigil:"::()" -> do
+        -- These are all parts of the name
+        parts   <- mapM fromVal =<< mapM evalExp exps
+        -- Now we only have to add the sigil in front of the string and join
+        -- the parts with "::".
+        let varname = sigil:(concat . (intersperse "::") $ parts)
+        -- Finally, eval the varname.
+        evalExp . Var $ varname
     "{}" -> do
         let [listExp, indexExp] = exps
         idxCxt  <- cxtOfExp indexExp 
@@ -515,13 +598,6 @@ reduce exp@(Syn name exps) = case name of
         doFetch (mkFetch $ doHash varVal hash_fetchElem)
                 (mkFetch $ doHash varVal hash_fetchVal)
                 (fromVal idxVal) lv (not (isSlurpyCxt idxCxt))
-    "try" -> do
-        val <- resetT $ evalExp (head exps)
-        retEvalResult False val
-    "gather" -> do
-        val     <- enterEvalContext cxtSlurpyAny exp
-        -- ignore val
-        retVal val
     "rx" -> do
         let [exp, adverbs] = exps
         hv      <- fromVal =<< evalExp adverbs
@@ -531,15 +607,17 @@ reduce exp@(Syn name exps) = case name of
         p5flags <- fromAdverb hv ["P5", "Perl5", "perl5"]
         flag_g  <- fromAdverb hv ["g", "global"]
         flag_i  <- fromAdverb hv ["i", "ignorecase"]
-        let rx | p5 = (`MkRulePCRE` g) . mkRegexWithPCRE (encodeUTF8 str) $
+        flag_s  <- fromAdverb hv ["stringify"] -- XXX hack
+        let rx | p5 = MkRulePCRE p5re g flag_s 
+               | otherwise = MkRulePGE str g flag_s
+            g = ('g' `elem` p5flags || flag_g)
+            p5re = mkRegexWithPCRE (encodeUTF8 str) $
                         [ pcreUtf8
                         , ('i' `elem` p5flags || flag_i) `implies` pcreCaseless
                         , ('m' `elem` p5flags) `implies` pcreMultiline
                         , ('s' `elem` p5flags) `implies` pcreDotall
                         , ('x' `elem` p5flags) `implies` pcreExtended
                         ]
-               | otherwise = MkRulePGE str g
-            g = ('g' `elem` p5flags || flag_g)
         retVal $ VRule rx
         where
         implies True  = id
@@ -573,20 +651,15 @@ reduce exp@(Syn name exps) = case name of
         let file = (`concatMap` mod) $ \v -> case v of
             { '-' -> "__"; _ | isAlphaNum v -> [v] ; _ -> "_" }
 #endif
-	externRequire "Haskell" (file ++ ".o")
+        externRequire "Haskell" (file ++ ".o")
         retEmpty
-    "~" -> do
-        let [exp] = exps
-        val     <- enterRValue $ enterEvalContext (cxtItem "Str") exp
-        if defined val
-            then retVal =<< fmap VStr (fromVal val)
-            else retVal undef
     syn | last syn == '=' -> do
         let [lhs, exp] = exps
             op = "&infix:" ++ init syn
         evalExp $ Syn "=" [lhs, App (Var op) [lhs, exp] []]
     _ -> retError "Unknown syntactic construct" exp
     where
+    doCond :: (Bool -> Bool) -> Eval Val
     doCond f = do
         let [cond, bodyIf, bodyElse] = exps
         vbool     <- enterEvalContext (cxtItem "Bool") cond
@@ -595,6 +668,7 @@ reduce exp@(Syn name exps) = case name of
             then reduce bodyIf
             else reduce bodyElse
     -- XXX This treatment of while/until loops probably needs work
+    doWhileUntil :: (Bool -> Bool) -> Eval Val
     doWhileUntil f = do
         let [cond, body] = exps
         enterLoop . fix $ \runLoop -> do
@@ -629,6 +703,13 @@ reduce (App (Var "&zip") invs args) = do
     val  <- op0Zip vals
     retVal val
 
+-- XXX absolutely evil bloody hack for "goto"
+reduce (App (Var "&not") [] []) = retEmpty
+
+reduce (App (Var "&not") invs args) = do
+    bool <- fromVal =<< evalExp (last $ invs ++ args)
+    retVal $ VBool (not bool)
+
 -- XXX absolutely evil bloody hack for "goto"
 reduce (App (Var "&goto") (subExp:invs) args) = do
     vsub <- enterEvalContext (cxtItem "Code") subExp
@@ -637,6 +718,7 @@ reduce (App (Var "&goto") (subExp:invs) args) = do
         val <- apply sub invs args
         shiftT $ const (retVal val)
     where
+    callerEnv :: Env -> Env
     callerEnv env = let caller = maybe env id (envCaller env) in
         env{ envCaller  = envCaller caller
            , envContext = envContext caller
@@ -658,8 +740,14 @@ reduce (App (Var name@('&':_)) invs args) = do
     sub     <- findSub name invs args
     case sub of
         Just sub    -> applySub sub invs args
-        Nothing     -> retError "No compatible subroutine found" name
+        _ | [Syn "," invs'] <- unwrap invs, null args -> do
+            sub <- findSub name invs' []
+            if isNothing sub then err else do
+            fail $ "Extra space found after " ++ name ++ " (...) -- did you mean " ++ name ++ "(...) instead?"
+        _ -> err
     where
+    err = retError "No compatible subroutine found" name
+    applySub :: VCode -> [Exp] -> [Exp] -> Eval Val
     applySub sub invs args
         -- list-associativity
         | MkCode{ subAssoc = "list" }      <- sub
@@ -680,6 +768,7 @@ reduce (App (Var name@('&':_)) invs args) = do
         -- normal application
         | otherwise
         = apply sub invs args
+    mungeChainSub :: VCode -> [Exp] -> Eval Val
     mungeChainSub sub invs = do
         let MkCode{ subAssoc = "chain", subParams = (p:_) } = sub
             (App (Var name') invs' args'):rest = invs
@@ -687,6 +776,7 @@ reduce (App (Var name@('&':_)) invs args) = do
         case theSub of
             Just sub'    -> applyChainSub sub invs sub' invs' args' rest
             Nothing      -> apply sub{ subParams = (length invs) `replicate` p } invs [] -- XXX Wrong
+    applyChainSub :: VCode -> [Exp] -> VCode -> [Exp] -> [a] -> [Exp] -> Eval Val
     applyChainSub sub invs sub' invs' args' rest
         | MkCode{ subAssoc = "chain", subBody = fun, subParams = prm }   <- sub
         , MkCode{ subAssoc = "chain", subBody = fun', subParams = prm' } <- sub'
@@ -705,8 +795,10 @@ reduce (App subExp invs args) = do
 
 reduce exp = retError "Invalid expression" exp
 
--- |Return the context that an expression bestows upon a hash or array
--- subscript. See 'reduce' for \{\} and \[\]. (Is this correct?)
+{-|
+Return the context that an expression bestows upon a hash or array
+subscript. See 'reduce' for \{\} and \[\]. (Is this correct?)
+-}
 cxtOfExp :: Exp -- ^ Expression to find the context of
          -> Eval Cxt
 cxtOfExp (Pos _ exp)            = cxtOfExp exp
@@ -714,6 +806,7 @@ cxtOfExp (Cxt cxt _)            = return cxt
 cxtOfExp (Syn "," _)            = return cxtSlurpyAny
 cxtOfExp (Syn "[]" [_, exp])    = cxtOfExp exp
 cxtOfExp (Syn "{}" [_, exp])    = cxtOfExp exp
+cxtOfExp (Syn (sigil:"{}") _) = return $ cxtOfSigil sigil
 cxtOfExp (Val (VList _))        = return cxtSlurpyAny
 cxtOfExp (Val (VRef ref))       = do
     cls <- asks envClasses
@@ -722,7 +815,7 @@ cxtOfExp (Val (VRef ref))       = do
         then cxtSlurpyAny
         else CxtItem typ
 cxtOfExp (Val _)                = return cxtItemAny
-cxtOfExp (Var (c:_))            = return $ cxtOfSigil c
+cxtOfExp (Var (sigil:_))            = return $ cxtOfSigil sigil
 cxtOfExp (App (Var "&list") _ _)   = return cxtSlurpyAny
 cxtOfExp (App (Var "&scalar") _ _) = return cxtSlurpyAny
 cxtOfExp (App (Var name) invs args)   = do
@@ -736,37 +829,106 @@ cxtOfExp (App (Var name) invs args)   = do
         _ -> cxtSlurpyAny
 cxtOfExp _                      = return cxtSlurpyAny
 
-findSub name [Var var] args = do
-    pos <- asks envPos
-    let findNext n = findSub n [Pos pos (Var var)] args -- XXX hack
-    rv  <- findVar var
-    case rv of
-        Nothing  -> findNext name
-        Just ref -> do
-            typ     <- evalValType (VRef ref)
-            subs    <- findNext $ ('&':showType typ) ++ "::" ++ tail name
-            if isNothing subs then findNext name else return subs
-findSub name [Val val] args = do
-    pos     <- asks envPos
-    let findNext n = findSub n [Pos pos (Val val)] args -- XXX hack
-    typ     <- evalValType val
-    subs    <- findNext $ ('&':showType typ) ++ "::" ++ tail name
-    if isNothing subs then findNext name else return subs
-findSub name [inv@(App (Var name') invs' args')] args = do
-    pos     <- asks envPos
-    let findNext n = findSub n [Pos pos inv] args -- XXX hack
-    sub     <- findSub name' invs' args'
-    case sub of
-        Just sub    -> do
-            let typ = subReturns sub
-            subs    <- findNext $ ('&':showType typ) ++ "::" ++ tail name
-            if isNothing subs then findNext name else return subs
-        Nothing     -> findNext name
-findSub name invs args = do
-    subSyms     <- findSyms name
-    lens        <- mapM argSlurpLen (unwrap $ invs ++ args)
-    doFindSub (sum lens) subSyms
+findSub :: String -> [Exp] -> [Exp] -> Eval (Maybe VCode)
+findSub name' invs args = do
+    let name = possiblyFixOperatorName name'
+    case invs of
+        [exp] | not (':' `elem` drop 2 name) -> do
+            typ     <- evalExpType exp
+            subs    <- findWithPkg (showType typ) name
+            if isJust subs then return subs else findSub' name
+        _ -> do
+            sub <- findSub' name
+            if isNothing sub then possiblyBuildMetaopVCode name else return sub
     where
+    possiblyBuildMetaopVCode op' | "&prefix:[" `isPrefixOf` op', "]" `isSuffixOf` op' = do 
+        -- Strip the trailing "]" from op
+        let op = drop 9 (init op')
+        -- We try to find the userdefined sub.
+        -- We use the first two elements of invs as invocants, as these are the
+        -- types of the op.
+            rv = findSub ("&infix:" ++ op) (take 2 (invs ++ [Val undef, Val undef])) []
+        maybeM rv $ \code -> return $ mkPrim
+            { subName     = "&prefix:[" ++ op ++ "]"
+            , subType     = SubPrim
+            , subAssoc    = "spre"
+            , subParams   = makeParams ["List"]
+            , subReturns  = mkType "Str"
+            , subBody     = Prim $ \[vs] -> do
+                list_of_args <- fromVal vs
+                op2Fold (list_of_args) (VCode code)
+            }
+        -- Now we construct the sub. Is there a more simple way to do it?
+    possiblyBuildMetaopVCode op' | "&prefix:" `isPrefixOf` op', "\171" `isSuffixOf` op' = do 
+        let op = drop 8 (init op')
+        possiblyBuildMetaopVCode ("&prefix:" ++ op ++ "<<")
+    possiblyBuildMetaopVCode op' | "&prefix:" `isPrefixOf` op', "<<" `isSuffixOf` op' = do 
+        let op = drop 8 (init (init op'))
+            rv = findSub ("&prefix:" ++ op) [head $ invs ++ [Val undef]] []
+        maybeM rv $ \code -> return $ mkPrim
+            { subName     = "&prefix:" ++ op ++ "<<"
+            , subType     = SubPrim
+            , subAssoc    = subAssoc code
+            , subParams   = subParams code
+            , subReturns  = mkType "List"
+            , subBody     = Prim
+                (\x -> op1HyperPrefix code (listArg x))
+            }
+    possiblyBuildMetaopVCode op' | "&postfix:\187" `isPrefixOf` op' = do
+        let op = drop 10 op'
+        possiblyBuildMetaopVCode ("&postfix:>>" ++ op)
+    possiblyBuildMetaopVCode op' | "&postfix:>>" `isPrefixOf` op' = do
+        let op = drop 11 op'
+            rv = findSub ("&postfix:" ++ op) [head $ invs ++ [Val undef]] []
+        maybeM rv $ \code -> return $ mkPrim
+            { subName     = "&postfix:>>" ++ op
+            , subType     = SubPrim
+            , subAssoc    = subAssoc code
+            , subParams   = subParams code
+            , subReturns  = mkType "List"
+            , subBody     = Prim
+                (\x -> op1HyperPostfix code (listArg x))
+            }
+    possiblyBuildMetaopVCode op' | "&infix:\187" `isPrefixOf` op', "\171" `isSuffixOf` op' = do 
+        let op = drop 8 (init op')
+        possiblyBuildMetaopVCode ("&infix:>>" ++ op ++ "<<")
+    possiblyBuildMetaopVCode op' | "&infix:>>" `isPrefixOf` op', "<<" `isSuffixOf` op' = do 
+        let op = drop 9 (init (init op'))
+            rv = findSub ("&infix:" ++ op) (take 2 (invs ++ [Val undef, Val undef])) []
+        maybeM rv $ \code -> return $ mkPrim
+            { subName     = "&infix:>>" ++ op ++ "<<"
+            , subType     = SubPrim
+            , subAssoc    = subAssoc code
+            , subParams   = makeParams ["Any", "Any"]
+            , subReturns  = mkType "List"
+            , subBody     = Prim (\[x, y] -> op2Hyper code x y)
+            }
+        -- Taken from Pugs.Prim. Probably this should be refactored. (?)
+    possiblyBuildMetaopVCode _ = return Nothing
+    listArg [x] = x
+    listArg xs = VList xs
+    makeParams = map (\p -> p{ isWritable = isLValue p }) . foldr foldParam [] . map takeWord
+    takeWord = takeWhile isWord . dropWhile (not . isWord)
+    isWord   = not . (`elem` "(),:")
+    findAttrs pkg = do
+        maybeM (findVar (':':'*':pkg)) $ \ref -> do
+            meta    <- readRef ref
+            fetch   <- doHash meta hash_fetchVal
+            fromVal =<< fetch "traits"
+    findWithPkg pkg name = do
+        subs <- findSub' (('&':pkg) ++ "::" ++ tail name)
+        if isJust subs then return subs else do
+        -- get superclasses
+        attrs <- findAttrs pkg
+        if isNothing attrs then findSub' name else do
+        (`fix` (fromJust attrs)) $ \run pkgs -> do
+            if null pkgs then return Nothing else do
+            subs <- findWithPkg (head pkgs) name
+            if isJust subs then return subs else run (tail pkgs)
+    findSub' name = do
+        subSyms     <- findSyms name
+        lens        <- mapM argSlurpLen (unwrap $ invs ++ args)
+        doFindSub (sum lens) subSyms
     argSlurpLen (Val listMVal) = do
         listVal  <- fromVal listMVal
         return $ length (vCast listVal :: [Val])
@@ -786,45 +948,50 @@ findSub name invs args = do
     subs slurpLen subSyms = (liftM catMaybes) $ (`mapM` subSyms) $ \(n, val) -> do
         sub@(MkCode{ subType = subT, subReturns = ret, subParams = prms }) <- fromVal val
         let isGlobal = '*' `elem` n
-        let fun = arityMatch sub (length (invs ++ args)) slurpLen
-        if isNothing fun then return Nothing else do
-        -- if deltaFromCxt ret == 0 then return Nothing else do
-        let invocants = filter isInvocant prms
-            prms' = if null invocants then prms else invocants
-            pairs = map (typeOfCxt . paramContext) prms'
-                        `zip` (map unwrap $ invs ++ args)
-        deltaCxt    <- deltaFromCxt ret
-        deltaArgs   <- mapM deltaFromPair pairs
-        let distance = (deltaCxt : deltaArgs)
-        let bound = either (const False) (const True) $ bindParams sub invs args
-        return $ Just
-            ( (isGlobal, subT, isMulti sub, bound, distance)
-            , fromJust fun
-            )
+        let rv = return $ arityMatch sub (length (invs ++ args)) slurpLen
+        maybeM rv $ \fun -> do
+            -- if deltaFromCxt ret == 0 then return Nothing else do
+            let pairs = map (typeOfCxt . paramContext) prms
+                            `zip` (map unwrap $ invs ++ args)
+            deltaCxt    <- deltaFromCxt ret
+            deltaArgs   <- mapM deltaFromPair pairs
+            let bound = either (const False) (const True) $ bindParams sub invs args
+            return ((isGlobal, subT, isMulti sub, bound, sum deltaArgs, deltaCxt), fun)
     deltaFromCxt x  = do
         cls <- asks envClasses
         cxt <- asks envContext
         return $ deltaType cls (typeOfCxt cxt) x
-    deltaFromScalar x = do
+    deltaFromPair (x, y) = do
         cls <- asks envClasses
-        return $ deltaType cls x (mkType "Scalar")
-    deltaFromPair (x, (Var var)) = do
-        cls <- asks envClasses
-        rv  <- findVar var
-        case rv of
-            Nothing  -> return $ deltaType cls x $ typeOfSigil (head var)
-            Just ref -> deltaFromPair (x, (Val $ VRef ref))
-    deltaFromPair (x, (Val val)) = do
-        cls <- asks envClasses
-        typ <- evalValType val
+        typ <- evalExpType y
         return $ deltaType cls x typ
-    deltaFromPair (x, _) = do
-        deltaFromScalar x
 
-
--- OK... Now let's implement the hideously clever autothreading algorithm.
--- First pass - thread thru all() and none()
--- Second pass - thread thru any() and one()
+evalExpType :: Exp -> Eval Type
+evalExpType (Var var) = do
+    rv  <- findVar var
+    case rv of
+        Nothing  -> return $ typeOfSigil (head var)
+        Just ref -> evalValType (VRef ref)
+evalExpType (Val val) = evalValType val
+evalExpType (App (Val val) _ _) = do
+    sub <- fromVal val
+    return $ subReturns sub
+evalExpType (App (Var "&new") [(Val (VType typ))] _) = return typ
+evalExpType (App (Var name) invs args) = do
+    sub <- findSub name invs args
+    case sub of
+        Just sub    -> return $ subReturns sub
+        Nothing     -> return $ mkType "Any"
+evalExpType exp@(Syn syn _) | (syn ==) `any` words "{} []" = do
+    val <- evalExp exp
+    evalValType val
+evalExpType (Cxt cxt _) | typeOfCxt cxt /= (mkType "Any") = return $ typeOfCxt cxt
+evalExpType (Cxt _ exp) = evalExpType exp
+evalExpType (Pos _ exp) = evalExpType exp
+evalExpType (Pad _ _ exp) = evalExpType exp
+evalExpType (Sym _ _ exp) = evalExpType exp
+evalExpType (Stmts _ exp) = evalExpType exp
+evalExpType _ = return $ mkType "Any"
 
 chainFun :: Params -> Exp -> Params -> Exp -> [Val] -> Eval Val
 chainFun p1 f1 p2 f2 (v1:v2:vs) = do
@@ -839,20 +1006,26 @@ chainFun _ _ _ _ _ = internalError "chainFun: Not enough parameters in Val list"
 
 applyExp :: SubType -> [ApplyArg] -> Exp -> Eval Val
 applyExp _ bound (Prim f) =
-    f [ argValue arg | arg <- bound, (argName arg !! 1) /= '_' ]
-applyExp _ [] body = evalExp body
-applyExp styp bound@(arg:_) body = do
+    f [ argValue arg | arg <- bound, (argName arg) /= "%_" ]
+applyExp styp bound body = applyThunk styp bound (MkThunk $ evalExp body)
+
+applyThunk :: SubType -> [ApplyArg] -> VThunk -> Eval Val
+applyThunk _ [] thunk = thunk_force thunk
+applyThunk styp bound@(arg:_) thunk = do
     -- introduce $?SELF and $_ as the first invocant.
     inv <- if styp <= SubMethod then invocant else return []
     pad <- formal
-    enterLex (inv ++ pad) $ evalExp body
+    enterLex (inv ++ pad) $ thunk_force thunk
     where
     formal = mapM argNameValue $ filter (not . null . argName) bound
     invocant = mapM (`genSym` (vCast $ argValue arg)) $ words "$?SELF $_"
     argNameValue (ApplyArg name val _) = genSym name (vCast val)
 
--- |Apply a sub (or other code) to lists of invocants and arguments.
--- Mostly delegates to 'doApply' after explicitly retrieving the local 'Env'.
+{-|
+Apply a sub (or other code) to lists of invocants and arguments.
+
+Mostly delegates to 'doApply' after explicitly retrieving the local 'Env'.
+-}
 apply :: VCode -- ^ The sub to apply
       -> [Exp] -- ^ List of invocants
       -> [Exp] -- ^ List of arguments (non-invocant)
@@ -863,14 +1036,16 @@ apply sub invs args = do
 
 -- XXX - faking application of lexical contexts
 -- XXX - what about defaulting that depends on a junction?
--- |Apply a sub (or other code) to lists of invocants
--- and arguments, in the specified context.
+{-|
+Apply a sub (or other code) to lists of invocants and arguments, in the 
+specified context.
+-}
 doApply :: Env   -- ^ Environment to evaluate in
         -> VCode -- ^ Code to apply
         -> [Exp] -- ^ Invocants (arguments before the colon)
         -> [Exp] -- ^ Arguments (not including invocants)
         -> Eval Val
-doApply env sub@MkCode{ subBody = fun, subType = typ } invs args =
+doApply env sub@MkCode{ subCont = cont, subBody = fun, subType = typ } invs args =
     case bindParams sub invs args of
         Left errMsg -> fail errMsg
         Right sub   -> do
@@ -886,14 +1061,18 @@ doApply env sub@MkCode{ subBody = fun, subType = typ } invs args =
                 -- trace (show bound) $ return ()
                 val <- local fixEnv $ enterLex syms $ do
                     (`juncApply` bound) $ \realBound -> do
-                        enterSub sub $ do
-                            applyExp (subType sub) realBound fun
+                        enterSub sub $ case cont of
+                            Just tvar   -> do
+                                thunk <- liftSTM $ readTVar tvar
+                                applyThunk (subType sub) realBound thunk
+                            Nothing     -> applyExp (subType sub) realBound fun
                 retVal val
     where
     enterScope :: Eval Val -> Eval Val
     enterScope
         | typ >= SubBlock = id
         | otherwise       = resetT
+    fixEnv :: Env -> Env
     fixEnv env
         | typ >= SubBlock = env
         | otherwise       = env
@@ -913,6 +1092,7 @@ doApply env sub@MkCode{ subBody = fun, subType = typ } invs args =
         newSym   <- genSym name boundRef
         (syms', restArgs) <- doBind (newSym:syms) rest
         return (syms', ApplyArg name val coll:restArgs)
+    expToVal :: Param -> Exp -> Eval (Val, Bool)
     expToVal MkParam{ isLazy = thunk, isLValue = lv, paramContext = cxt, paramName = name, isWritable = rw } exp = do
         env <- ask -- freeze environment at this point for thunks
         let eval = local (const env{ envLValue = lv }) $ do
@@ -924,7 +1104,12 @@ doApply env sub@MkCode{ subBody = fun, subType = typ } invs args =
             if isaType cls "Junction" typ then return v else do
             case (lv, rw) of
                 (True, True)    -> return v
-                (True, False)   -> return (VRef $ scalarRef v) 
+                (True, False)   -> do
+                    --- not scalarRef!
+                    case showType (typeOfSigil $ head name) of
+                        "Hash"  -> fmap (VRef . hashRef) (fromVal v :: Eval VHash)
+                        "Array" -> fmap (VRef . arrayRef) (fromVal v :: Eval VArray)
+                        _       -> return (VRef $ scalarRef v) 
                 (False, False)  -> return v -- XXX reduce to val?
                 (False, True)   -> do
                     -- make a copy
@@ -932,16 +1117,19 @@ doApply env sub@MkCode{ subBody = fun, subType = typ } invs args =
                     writeRef ref v
                     return (VRef ref)
         return (val, (isSlurpyCxt cxt || isCollapsed (typeOfCxt cxt)))
+    checkSlurpyLimit :: (VInt, Exp) -> Eval [Val]
     checkSlurpyLimit (n, exp) = do
         listVal <- enterLValue $ enterEvalContext (cxtItem "Array") exp
         list    <- fromVal listVal
         elms    <- mapM fromVal list -- flatten
         return $ genericDrop n (concat elms :: [Val])
+    isCollapsed :: Type -> Bool
     isCollapsed typ
         | isaType (envClasses env) "Bool" typ        = True
         | isaType (envClasses env) "Junction" typ    = True
         | otherwise                     = False
 
+toGlobal :: String -> String
 toGlobal name
     | (sigil, identifier) <- break (\x -> isAlpha x || x == '_') name
     , last sigil /= '*'
@@ -949,18 +1137,19 @@ toGlobal name
     | otherwise = name
 
 
+arityMatch :: VCode -> Int -> Int -> Maybe VCode
 arityMatch sub@MkCode{ subAssoc = assoc, subParams = prms } argLen argSlurpLen
     | assoc == "list" || assoc == "chain"
     = Just sub
     | isNothing $ find (not . isSlurpy) prms -- XXX - what about empty ones?
     , assoc == "pre"
     , slurpLen <- length $ filter (\p -> isSlurpy p && head (paramName p) == '$') prms
-    , hasArray <- isJust $ find (\p -> isSlurpy p && head (paramName p) == '@') prms
+    , hasArray <- isJust $ find (\p -> isSlurpy p && head (paramName p) /= '$') prms
     , if hasArray then slurpLen <= argSlurpLen else slurpLen == argSlurpLen
     = Just sub
     | reqLen <- length $ filter (\p -> not (isOptional p || isSlurpy p)) prms
     , optLen <- length $ filter (\p -> isOptional p) prms
-    , hasArray <- isJust $ find (\p -> isSlurpy p && head (paramName p) == '@') prms
+    , hasArray <- isJust $ find (\p -> isSlurpy p && head (paramName p) /= '$') prms
     , argLen >= reqLen && (hasArray || argLen <= (reqLen + optLen))
     = Just sub
     | otherwise
