@@ -20,6 +20,12 @@ import Pugs.Prim
 import Pugs.Embed
 import qualified Data.Map as Map
 
+{-|
+Run 'Main.run' with command line args. 
+
+See 'Main.main' and 'Pugs.Run.Args.canonicalArgs'
+-}
+runWithArgs :: ([String] -> IO t) -> IO t
 runWithArgs f = do
     args <- getArgs
     f $ canonicalArgs args
@@ -34,6 +40,7 @@ runEvalMain env eval = withSocketsDo $ do
 runEnv :: Env -> IO Val
 runEnv env = runEvalMain env $ evaluateMain (envBody env)
 
+-- | Run for 'Pugs.Compile.Pugs' backend
 runAST :: Pad -> Exp -> IO Val
 runAST glob ast = do
     hSetBuffering stdout NoBuffering
@@ -45,6 +52,7 @@ runAST glob ast = do
         newTVar (glob `unionPads` glob')
     runEnv env{ envBody = ast, envGlobal = globRef, envDebug = Nothing }
 
+-- | Run for 'Pugs.Compile.Haskell' backend
 runComp :: Eval Val -> IO Val
 runComp comp = do
     hSetBuffering stdout NoBuffering
@@ -53,6 +61,7 @@ runComp comp = do
     env  <- prepareEnv name args
     runEvalMain env{ envDebug = Nothing } comp
 
+-- | Initialize globals and install primitives in an 'Env'
 prepareEnv :: VStr -> [VStr] -> IO Env
 prepareEnv name args = do
     let confHV = Map.map VStr config
@@ -70,6 +79,8 @@ prepareEnv name args = do
     egidSV  <- newScalar (VInt $ toInteger egid)
     execSV  <- newScalar (VStr exec)
     progSV  <- newScalar (VStr name)
+    checkAV <- newArray []
+    initAV  <- newArray []
     endAV   <- newArray []
     matchAV <- newScalar (VMatch mkMatchFail)
     incAV   <- newArray (map VStr libs)
@@ -100,6 +111,8 @@ prepareEnv name args = do
         , genSym "$*EUID"       $ MkRef euidSV
         , genSym "$*GID"        $ MkRef gidSV
         , genSym "$*EGID"       $ MkRef egidSV
+        , genSym "@?CHECK"      $ MkRef checkAV
+        , genSym "@?INIT"       $ MkRef initAV
         , genSym "@*END"        $ MkRef endAV
         , genSym "$*IN"         $ MkRef inGV
         , genSym "$*OUT"        $ MkRef outGV
@@ -122,6 +135,10 @@ prepareEnv name args = do
         , genSym "$*_" $ MkRef defSV
         ]
 
+{-|
+Combine @%*ENV\<PERL6LIB\>@, -I, 'Pugs.Config.config' values and \".\" into
+the @\@*INC@ list for 'Main.printConfigInfo'
+-}
 getLibs :: IO [String]
 getLibs = do
     args    <- getArgs
