@@ -9,6 +9,7 @@ class (Typeable a) => CodeClass a where
     code_apply    :: a -> Eval Val
     code_assoc    :: a -> VStr
     code_params   :: a -> Params
+    code_type     :: a -> SubType
 
 instance CodeClass ICode where
     code_iType c  = code_iType . unsafePerformSTM $ readTVar c
@@ -19,14 +20,14 @@ instance CodeClass ICode where
     code_apply    = error "apply"
     code_assoc c  = code_assoc . unsafePerformSTM $ readTVar c
     code_params c = code_params . unsafePerformSTM $ readTVar c
+    code_type c   = code_type . unsafePerformSTM $ readTVar c
 
 instance CodeClass VCode where
     -- XXX - subType should really just be a mkType itself
     code_iType c  = case subType c of
         SubBlock    -> mkType "Block"
-        SubRoutine  -> mkType "Sub"
-        SubPrim     -> mkType "Sub"
         SubMethod   -> mkType "Method"
+        _           -> mkType "Sub"
     code_fetch    = return
     code_store _ _= retConstError undef
     code_assuming c [] [] = return c
@@ -34,4 +35,5 @@ instance CodeClass VCode where
     code_apply    = error "apply"
     code_assoc    = subAssoc
     code_params   = subParams
+    code_type     = subType
 
